/*
 *	(c) copyright 2003-2004 Amichai Rothman
 *
 *	This file is part of the Java TNEF package.
 *
 *	The Java TNEF package is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	The Java TNEF package is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.freeutils.tnef;

/**
 * The <code>MAPIProp</code> class encapsulates the name of a named MAPI property.
 * A MAPI Property name consists of a GUID, as well as either a 32-bit identifier
 * or a String.
 *
 * @author  Amichai Rothman
 * @since	2003-07-25
 */
public class MAPIPropName {

	/**
	 * MAPI property name type constant.
	 */
	public static final int
		MNID_ID		= 0,
		MNID_STRING	= 1;

	byte[]	GUID;
	int		type;
	long	ID;
	String	name;
	int rawLength;

	/**
	 * Constructs a MAPIPropName using the given TNEF stream bytes.
	 * 
	 * @param data the TNEF stream bytes containing the property name data
	 * @param offset the offset within data where the property name data starts
	 */
	public MAPIPropName(byte[] data, int offset) {
		// get TRP structure values
		this.GUID = new byte[16];
		System.arraycopy(data,offset,GUID,0,16);
		this.type = (int)TNEFUtils.getU32(data,offset + 16);
		this.rawLength = 20;
		if (type == MNID_STRING) {
			int length = (int)TNEFUtils.getU32(data,offset + 20);
			this.rawLength += 4;
			this.name = TNEFUtils.createStringUnicode(data,offset + 24, length);
			if (length % 4 != 0) // pad to 4 byte boundary
				length += (4 - (length % 4));
			this.rawLength += length;
		} else if (type == MNID_ID) {
			this.ID = TNEFUtils.getU32(data,offset + 20);
			this.rawLength += 4;
		}
	}

	/**
	 * Constructs a MAPIPropName containing given values.
	 * 
	 * @param GUID the property GUID
	 * @param ID the property ID
	 */
	public MAPIPropName(byte[] GUID, long ID) {
		this.GUID = GUID;
		this.type = MNID_ID;
		this.ID = ID;
	}

	/**
	 * Constructs a MAPIPropName containing given values
	 * 
	 * @param GUID the property GUID
	 * @param name the property name
	 */
	public MAPIPropName(byte[] GUID, String name) {
		this.GUID = GUID;
		this.type = MNID_STRING;
		this.name = name;
	}

	/**
	 * Gets the MAPIPropName GUID.
	 * 
	 * @return the MAPIPropName GUID.
	 */
	public byte[] getGUID() {
		return this.GUID;
	}

	/**
	 * Gets the MAPIPropName type.
	 * 
	 * @return the MAPIPropName type.
	 */
	public int getType() {
		return this.type;
	}

	/**
	 * Gets the MAPIPropName ID.
	 * 
	 * @return the MAPIPropName ID.
	 */
	public long getID() {
		return this.ID;
	}

	/**
	 * Gets the MAPIPropName name.
	 * 
	 * @return the MAPIPropName name.
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * Returns a string representation of the object.
	 * 
	 * @return a string representation of the object.
	 */
	public String toString() {
		StringBuffer s = new StringBuffer()
			.append("GUID=")
			.append(TNEFUtils.toHexString(GUID))
			.append(" Name=")
			.append(type == MNID_STRING ? name : "0x"+Long.toHexString(ID));
		return s.toString();
	}

	/**
	 * Gets the raw data length (in bytes) of this instance.
	 * 
	 * @return the raw data length (in bytes) of this instance.
	 */
	protected int getRawLength() {
		return this.rawLength;
	}

}
