/*
 *	(c) copyright 2003-2004 Amichai Rothman
 *
 *	This file is part of the Java TNEF package.
 *
 *	The Java TNEF package is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 2 of the License, or
 *	(at your option) any later version.
 *
 *	The Java TNEF package is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program; if not, write to the Free Software
 *	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package net.freeutils.tnef;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Date;

/**
 * The <code>MAPIProp</code> class encapsulates the name of a named MAPI property.
 * A MAPI Property name consists of a GUID, as well as either a 32-bit identifier
 * or a String.
 *
 * @author  Amichai Rothman
 * @since	2003-07-25
 */
public class MAPIValue {

	int type;
    byte[] data;

	/**
	 * Constructs a MAPIValue containing given values.
	 * 
	 * @param type the value type
	 * @param buf the TNEF bytes containing the value
	 * @param offset the offset into buf where the value data starts
	 * @param length the length of the value data (in bytes)
	 */
	public MAPIValue(int type, byte[] buf, int offset, int length) {
		this.type = type;
		this.data = new byte[length];
		System.arraycopy(buf,offset,this.data,0,length);
	}

	/**
	 * Constructs a MAPIValue containing given values.
	 * 
	 * @param type the value type
	 * @param buf the TNEF bytes containing the value
	 */
	public MAPIValue(int type, byte[] buf) {
		this.type = type;
		this.data = buf;
	}

	/**
	 * Gets the MAPIValue type.
	 * 
	 * @return the MAPIValue type.
	 */
	public int getType() {
		return this.type;
	}

	/**
	 * Gets the MAPIValue data length (in bytes).
	 * 
	 * @return the MAPIValue data length (in bytes).
	 */
	public int getLength() {
		return this.data == null ? 0 : this.data.length;
	}

	/**
	 * Gets the MAPIValue data.
	 * 
	 * @return the MAPIValue data.
	 */
	public byte[] getData() {
		return this.data;
	}

	/**
	 * Returns a string representation of this object.
	 * 
	 * @return a string representation of this object.
	 */
	public String toString() {
		try {
			Object o = getValue();
			if (o instanceof ByteArrayInputStream) {
				byte[] bytes = new byte[((ByteArrayInputStream)o).available()];
				((ByteArrayInputStream)o).read(bytes);
				o = TNEFUtils.toHexString(bytes,512);
			} else if (o instanceof byte[]) {
				o = TNEFUtils.toHexString((byte[])o,512);
			}
			return String.valueOf(o);
		} catch (IOException ioe) {
			throw new RuntimeException(ioe.toString());
		}
	}

	/**
	 * Returns the value encapsulated by this MAPIValue.
	 * The returned Object should be cast into an appropriate class,
	 * according to the MAPIValue's type.
	 * 
	 * @return the value encapsulated by this MAPIValue.
	 */
	public Object getValue() throws IOException {
		Object o = null;
		switch (type) {
				case MAPIProp.PT_NULL:
					o = null;
					break;

				case MAPIProp.PT_SHORT:
					// 2 bytes
					o = new Short((short)TNEFUtils.getU16(data[0],data[1]));
					break;

				case MAPIProp.PT_INT:
					// 4 bytes
					o = new Integer((int)TNEFUtils.getU32(data,0));
					break;

				case MAPIProp.PT_BOOLEAN:
					// 4 bytes
					o = new Boolean(TNEFUtils.getU32(data,0) != 0);
					break;

				case MAPIProp.PT_FLOAT:
					// 4 bytes
					o = new Float(Float.intBitsToFloat((int)TNEFUtils.getU32(data,0)));
					break;

				case MAPIProp.PT_ERROR:
					// 4 bytes
					o = new Integer((int)TNEFUtils.getU32(data,0));
					break;

				case MAPIProp.PT_APPTIME:
				case MAPIProp.PT_SYSTIME:
					// 64-bit Windows FILETIME is 100ns since January 1, 1601 (UTC)
					if (data.length == 8) {
						long time = TNEFUtils.getU64(data,0);
						time = time/10/1000; // to milliseconds
						time = time - 1000L*60L*60L*24L*(365L*369L+89L); // milliseconds between 1/1/1601 and 1/1/1970 (including 89 leap year days)
						o = new Date(time);
					}
					break;

				case MAPIProp.PT_DOUBLE:
					// 8 bytes
					o = new Double(Double.longBitsToDouble(TNEFUtils.getU64(data,0)));
					break;

				case MAPIProp.PT_CURRENCY:
				case MAPIProp.PT_INT8BYTE:
					// 8 bytes
					o = new long[] {TNEFUtils.getU32(data,0),TNEFUtils.getU32(data,4)};
					break;

				case MAPIProp.PT_CLSID:
					// CLSID - 20 bytes
					o = data;
				  break;

				case MAPIProp.PT_STRING:
					// variable length string
					o = TNEFUtils.createString(data,0,data.length);
					break;

				case MAPIProp.PT_UNICODE_STRING:
					// variable length string
					o = TNEFUtils.createStringUnicode(data,0,data.length);
					break;

				case MAPIProp.PT_BINARY:
				case MAPIProp.PT_UNSPECIFIED:
					// variable length bytes
					o = data;
					break;

				case MAPIProp.PT_OBJECT:
					// get object IID
					byte[] IID = new byte[16];
					System.arraycopy(data,0,IID,0,16);
					o = new ByteArrayInputStream(data,16,data.length-16);
					if (TNEFUtils.equals(IID,0,MAPIProp.IID_IMESSAGE,0,IID.length))
						o = new TNEFInputStream((ByteArrayInputStream)o);

					break;
				default:
					throw new IOException("Unknown MAPI type: " + type);
		}
		return o;
	}

}
